<?php

if(!class_exists('WC_Payment_Gateway')) return;

class WC_PK_Gateway extends WC_Payment_Gateway {

    protected $order = null;
    protected $transactionId = null;
    protected $transactionErrorMessage = null;
    protected $server = '';
    protected $secret = '';
	
	/* инициализация плагина */
    public function __construct() {
        
		$this->id = 'paykeeper';
        $this->GATEWAYNAME = 'PayKeeper';
        $this->has_fields = false;

		load_plugin_textdomain('paykeeper', false, dirname(plugin_basename(__FILE__)) . '/languages/');
		
        $this->init_form_fields();
        $this->init_settings();

		$this->title = $this->settings['title'];
		$this->icon = $this->settings['icon'];
        $this->description = $this->settings['description'];
        $this->server = $this->settings['paykeeperserver'];
        $this->secret = $this->settings['paykeepersecret'];        
        
        add_action('admin_notices', array(&$this, 'handle_admin_notice_msg'));
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
       
		add_action('woocommerce_receipt_' . $this->id, array( $this, 'receipt_page' ) );
        add_action('woocommerce_api_wc_pk_gateway', array( $this, 'paykeeper_handler' ) );
    }
	
	function handle_admin_notice_msg() {}
	
	/* Выводим настройки в административной панели*/
    public function admin_options() {
        ?>
        <h3><?php _e('PayKeeper', 'paykeeper'); ?></h3>
        <p><?php _e('Allows Payments via the PayKeeper gateway.', 'paykeeper'); ?></p>
        
        <table class="form-table">
            <?php 
            $this->generate_settings_html();
			?>
        </table>
        <?php
    }
    
	/* набор настроек */
    public function init_form_fields() {
        
		$this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable/Disable', 'woocommerce'),
                'type' => 'checkbox',
                'label' => __('Enable PayKeeper Gateway', 'paykeeper'),
                'default' => 'yes'
            ),
            'paykeeperserver' => array(
                'title' => __('PayKeeper server URL', 'paykeeper'),
                'type' => 'text',
                'description' => __('Your PayKeeper server URL.', 'paykeeper'),
                'default' => __('', 'paykeeper')
            ),
            'paykeepersecret' => array(
                'title' => __('PayKeeper secret word', 'paykeeper'),
                'type' => 'text',
                'description' => __('Your PayKeeper secret word.', 'paykeeper'),
                'default' => __('', 'paykeeper')
            ),
			'title' => array(
                'title' => __('Title', 'woocommerce'),
                'type' => 'text',
                'description' => __('The title for this checkout option.', 'woocommerce'),
                'default' => __('PayKeeper Payment', 'paykeeper')
            ),
			'description' => array(
                'title' => __('Description', 'woocommerce'),
                'type' => 'text',
                'description' => __('The description for this checkout option.', 'woocommerce'),
                'default' => __('Payments via the PayKeeper gateway', 'paykeeper')
            ),
			'icon' => array(
                'title' => __('Icon', 'woocommerce'),
                'type' => 'text',
                'description' => __('The icon for this checkout option.', 'woocommerce'),
                'default' => __('/wp-content/plugins/woocommerce-paykeeper/visa_mastercard_wordpress.png', 'paykeeper')
            ),
			'notify_link' => array(
                'title' => __('Notify link', 'paykeeper'),
                'type' => 'text',
                'custom_attributes' => array('readonly' => 'readonly'),
                'description' => __('Copy and paste this link to your config at PayKeeper account as notify link for server-server notification.', 'paykeeper'),
                'default' => __(home_url().'/?wc-api=wc_pk_gateway', 'paykeeper')
            ),
			'success_link' => array(
                'title' => __('Success link', 'paykeeper'),
                'type' => 'text',
                'custom_attributes' => array('readonly' => 'readonly'),
                'description' => __('Copy and paste this link to your config at PayKeeper account as success payment redirect.', 'paykeeper'),
                'default' => __(home_url().'/?wc-api=wc_pk_gateway', 'paykeeper')
            ),
			'failed_link' => array(
                'title' => __('Failed link', 'paykeeper'),
                'type' => 'text',
                'custom_attributes' => array('readonly' => 'readonly'),
                'description' => __('Copy and paste this link to your config at PayKeeper account as failed payment redirect.', 'paykeeper'),
                'default' => __(home_url().'/?wc-api=wc_pk_gateway&failed', 'paykeeper')
            ),
        );
    }

	/* вывод описания платежной системы в списке доступных платежей на странице оформления заказа */
	function payment_fields(){
        if($this -> description) echo wpautop(wptexturize($this -> description));
    }
	
	/* обработка заказа, очистка корзины, переход на страницу оплаты */
    public function process_payment($order_id) {
       
		$order = new WC_Order($order_id);

		global $woocommerce;
		$woocommerce->cart->empty_cart();
		$order->reduce_order_stock();
		
		return array(
			'result' => 'success',
			'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, $order->get_checkout_payment_url(true)))
		);
    }
	
	/* обработка серверного-ответа на стороне сайта о подтверждении платежа */
	public function paykeeper_handler() {
		if ( ! empty( $_POST ) ) {
			
			/* у нас магазин и мы не можем обрабатывать запросы без номера заказа, хотя orderid и не обязателен */
			$order_id = $_POST['orderid'];
			if ($order_id == ""){
				echo "Error! Order id is missing";
				exit;
			}
			
			$id = $_POST['id'];
			$key = $_POST['key'];
			$email = $_POST['clientid'];
			$sum = $_POST['sum'];

			/* проверка цифровой подписи */
			if ($key != md5 ($id . $sum.$email.$order_id.$this->secret)){
				echo "Error! Hash mismatch";
				exit;
			}
			
			$order = new WC_Order($order_id);
			
			/* если заказ уже оплачен, то уведомляем, что все хорошо */
			if ( $order->has_status( 'completed' ) ) {
				echo "OK ".md5($id.$this->secret);
				exit;
			}

			/* платеж успешно совершен */
			$order->payment_complete();
			$order->add_order_note("PayKeeper payment completed");
			unset($_SESSION['order_awaiting_payment']);
			
			echo "OK ".md5($id.$this->secret);
			exit;
		}elseif(isset($_REQUEST['failed'])) self::paykeeper_failed();
		else self::paykeeper_success();
	}
	
	/* что делать при ошибке платежа */
	public function paykeeper_failed(){
		wp_redirect( get_permalink( wc_get_page_id( 'cart' ) ) );
	}
	
	/* при успешном платеже отправляем пользователя на страницу его аккаунта, последний оплаченный заказ будет самым первым в списке */
	public function paykeeper_success(){
		wp_redirect( get_permalink( wc_get_page_id( 'myaccount' ) ) );
	}
	
	/* вывод формы на странице оплаты */
 	function receipt_page($order_id){
		
		$order = new WC_Order( $order_id );
		
		$payment_parameters = http_build_query(array(
			"orderid"=>$order->id,
			"clientid"=>$order->billing_email?$order->billing_email:'anonymous',
			"sum"=>$order->get_total(),
			"phone"=>$order->billing_phone
		));

		$options = array("http"=>array(
			"method"=>"POST",
			"header"=>
			"Content-type: application/x-www-form-urlencoded",
			"content"=>$payment_parameters
		));
		
		$context = stream_context_create($options);
		
		echo file_get_contents($this->server,FALSE, $context);
	}

}
